///////////////////////////////////////////////////////////////////////////////
//                         Configure the pins location                       //
///////////////////////////////////////////////////////////////////////////////
#define  INT1_PIN       PIN_B0   //Interruption 1
#define  INT2_PIN       PIN_B1   //Interruption 2
#define  SCL_PIN        PIN_B2   //S Clock
#define  SDA_PIN        PIN_B3   //S Data
#define  DR_DEFAULT     DR_800   //DR used for the embedded channels,
                                 //[800] default
                                 //DR_x : 800, 200, 100, 50, 12_5, 6_25 or 1_56
                                 
///////////////////////////////////////////////////////////////////////////////
//                         Configure if using tap detection                  //
///////////////////////////////////////////////////////////////////////////////
#define  X_G            3        
#define  Y_G            3
#define  Z_G            3
//The counts for each register, 800 Hz ODR and normal mode (DEFAULT)
//For more information refer to the MMA8452Q datasheet
#define  TMLT_COUNT     96       // [60ms] / [0.625ms] = 96
#define  LTCY_COUNT     160      // [200ms] / [1.25ms] = 160
#define  WIND_COUNT     160      // [200ms] / [1.25ms] = 160

///////////////////////////////////////////////////////////////////////////////
//                         I2C CONFIGURATION                                 //
///////////////////////////////////////////////////////////////////////////////
#use     I2C(MASTER, SCL=SCL_PIN, SDA=SDA_PIN)
#define  DIR_ESCLAVO_W     0x3A  //Slave address for writing [8 bits]
#define  DIR_ESCLAVO_R     0x3B  //Slave address for reading[8 bits]


///////////////////////////////////////////////////////////////////////////////
//  This library allows the user to write and read the MMA8452Q registers,   //
//  also use the embedded functions of the sensor.                           //
//                                                                           //
//  It is important to configure the pins of the library to fit on your own  //
//  connections, this can be done by editing the MMA8452Q.h file, also the   //
//  user has to calculate some constants that depend on the ODR selected     //
//  and the sensor mode.                                                     //
//                                                                           //
/////////////////////////  INITIALIZATION   ///////////////////////////////////
//                                                                           //
//  Before using the library, the user has to implement the following        //
//  function:                                                                //
//                                                                           //
//       void MMA8452_init(int32 mask) : It will initialize the sensor       //
//                configuring the Control Registers.                         //
//                Refer to the constants used on MMA8452_init() function     //
//                                                                           //
/////////////////////////////  REGISTERS  /////////////////////////////////////
//                                                                           //
//  The user can write or read any register implementing the following       //
//  functions:                                                               //
//                                                                           //
//       void writeReg(unsigned int reg, unsigned int data) : Write the      //
//                   "data" into the "reg", using I2C.                       //
//                                                                           //
//       int readReg(unsigned int reg) : Read the specified register using   //
//                   I2C                                                     //
//                                                                           //
////////////////////////////// IT IS CONNECTED? ///////////////////////////////
//                                                                           //
//  The user can check if the device is connected with this function:        //
//                                                                           //
//       int MMA8452_is_connected() : It will read the who am I register     //
//                   and compared the information received by I2C with the   //
//                   information of the sensor.                              //
//               Return MMA8452_CONNECTED : If is connected                  //
//               Return MMA8452_NOT_CONNECTED : If is not connected          //
//                                                                           //
//////////////////////////// X-AXIS Y-AXIS Z-AXIS  ////////////////////////////
//                                                                           //
//  For read and configure the X-axis, Y-axis and Z-axis of the sensor       //
//                                                                           //
//       void MMA8452_xyz_conf(int mask) : Configure the scale range and     //
//                  the high pass filter                                     //
//                Refer to Constants used for MMA8452_xyz_conf();            //
//                                                                           //
//       signed long int MMA8452_x_read() : Return the X-axis value          //
//                                                                           //
//       signed long int MMA8452_y_read() : Return the Y-axis value          //
//                                                                           //
//       signed long int MMA8452_z_read() : Return the Z-axis value          //
//                                                                           //
/////////////////////////////   ORIENTATION  //////////////////////////////////
//                                                                           //
//  For use the Orientation (Portland / Landscape) detection you hace to     //
//  enable it implementing the MMA8452_enPortLand() function                 //
//                                                                           //
//       void MMA8452_enPortLand() : Enable the orientation detection        //
//                                                                           //
//       int MMA8452_newOrientation() :                                      //
//     Return MMA8452_ORIEN_CHANGED : If the orientation changed             //
//     Return MMA8452_ORIEN_NOT_CHANGED : If the orientation did not changed //
//                                                                           //
//       int MMA8452_LaPo() :                                                //
//                Return MMA8452_PORTRAIT_UP : Portrait Up                   //
//                Return MMA8452_PORTRAIT_DOWN : Portrait Down               //
//                Return MMA8452_LANDSCAPE_RIGHT : Landscape Right           //
//                Return MMA8452_LANDSCAPE_LEFT : Landscape Left             //
//                                                                           //
//       int MMA8452_BaFro() :                                               //
//                Return MMA8452_FRONT_FACE : Front facing orientation       //
//                Return MMA8452_BACK_FACE : Back facing orientation         //
//                                                                           //
//////////////////////////////   TAP DETECTION  ///////////////////////////////
//                                                                           //
// For use the tap detection, the user has to enable this embedded function  //
// implementing the MMA8452_enTapDet() function:                             //
//                                                                           //
//       void MMA8452_enTapDet(int mask) : The user has to select if is      //
//                   going to use Single or/and Double Tap detection.        //
//                   Refer to Constants used for MMA8452_enTapDet();         //
//                                                                           //
//       int MMA8452_getTap() :                                              //
//                Return MMA8452_NO_PULSE : If not tap detected              //
//                Return MMA8452_DOUBLE_PULSE : If double tap detected       //
//                Return MMA8452_SINGLE_PULSE : If single tap detecetd       //
///////////////////////////////////////////////////////////////////////////////

#define  MMA8452_CONNECTED          1
#define  MMA8452_NOT_CONNECTED      0
#define  MMA8452_ORIEN_CHANGED      1
#define  MMA8452_ORIEN_NOT_CHANGED  0
#define  MMA8452_PORTRAIT_UP        0
#define  MMA8452_PORTRAIT_DOWN      1
#define  MMA8452_LANDSCAPE_RIGHT    2  
#define  MMA8452_LANDSCAPE_LEFT     3
#define  MMA8452_FRONT_FACE         0
#define  MMA8452_BACK_FACE          1
#define  MMA8452_NO_PULSE           0
#define  MMA8452_DOUBLE_PULSE       1
#define  MMA8452_SINGLE_PULSE       2

///////////////////////////////////////////////////////////////////////////////
//                         Register Address                                  //
///////////////////////////////////////////////////////////////////////////////
#define  OUT_X_MSB         0x01
#define  OUT_X_LSB         0x02
#define  OUT_Y_MSB         0x03
#define  OUT_Y_LSB         0x04
#define  OUT_Z_MSB         0x05
#define  OUT_Z_LSB         0x06
#define  SYSMOD            0x0B
#define  INT_SOURCE        0x0C
#define  WHO_AM_I          0x0D
#define  XYZ_DATA_CFG      0x0E
#define  HP_FILTER_CUTOFF  0x0F
#define  PL_STATUS         0x10
#define  PL_CFG            0x11
#define  PL_COUNT          0x12
#define  PL_BF_ZCOMP       0x13
#define  P_L_THS_REG       0x14
#define  FF_MT_CFG         0x15
#define  FF_MT_SRC         0x16
#define  FF_MT_THS         0x17
#define  FF_MT_COUNT       0x18
#define  PULSE_CFG         0x21
#define  PULSE_SRC         0x22
#define  PULSE_THSX        0x23
#define  PULSE_THSY        0x24
#define  PULSE_THSZ        0x25
#define  PULSE_TMLT        0x26
#define  PULSE_LTCY        0x27
#define  PULSE_WIND        0x28
#define  CTRL_REG1         0x2A
#define  CTRL_REG2         0x2B
#define  CTRL_REG3         0x2C
#define  CTRL_REG4         0x2D
#define  CTRL_REG5         0x2E


///////////////////////////////////////////////////////////////////////////////
//                    Constants used for MMA8452_init() are:                 //
///////////////////////////////////////////////////////////////////////////////
//       Default initialization values:                                      //
//          ASLP Rate: 50Hz                                                  //
//          Data rate: 800Hz                                                 //
//          Noise: Normal                                                    //
//          Fast Read: Not enable                                            //
//          Normal mode                                                      //
//          No Autosleep                                                     //
//          No software reset                                                //
//          No selftest enabled                                              //
//          Push Pull                                                        //
//          IPOL low                                                         //
//          No interrupts                                                    //
///////////////////////////////////////////////////////////////////////////////
//Auto wake up sample frequency when device is in sleep mode
#define  ASLP_RATE_50      0x00000000     // 50Hz
#define  ASLP_RATE_12_5    0x00000040     // 12.5 Hz
#define  ASLP_RATE_6_25    0x00000080     // 6.25 Hz
#define  ASLP_RATE_1_56    0x000000C0     // 1.56 Hz
//Data rate selection
#define  DR_800            0x00000000     // 800 Hz
#define  DR_400            0x00000008     // 400 Hz
#define  DR_200            0x00000010     // 200 Hz
#define  DR_100            0x00000018     // 100 Hz
#define  DR_50             0x00000020     // 50 Hz
#define  DR_12_5           0x00000028     // 12.5 Hz
#define  DR_6_25           0x00000030     // 6.25 Hz
#define  DR_1_56           0x00000038     // 1.56 Hz

#define  NOISE_NORMAL      0x00000000     // Noise normal
#define  NOISE_RED         0x00000004     // Reduced noise

#define  NO_FAST_READ      0x00000000     // Normal mode
#define  FAST_READ         0x00000002     // Fast read mode

//Power mode
#define  NORMAL_MODE       0x00000000     // Normal
#define  LOW_NOISEPOWER    0x00000100     // Low noise low power
#define  HIGH_RES          0x00000200     // High resolution
#define  LOW_POWER         0x00000300     // Low power

#define  NO_AUTOSLEEP      0x00000000     // Autosleep is not enabled
#define  AUTOSLEEP         0x00000400     // Autosleep is enabled

#define  SW_RST            0x00000800     // Software reset enabled
#define  NO_SWR            0x00000000     // Software reset disabled

#define  SELFTEST          0x00001000     // Self test enabled
#define  NO_SELFTEST       0x00000000     // Self test disabled

#define  PUSH_PULL         0x00000000     // Push pull
#define  OPEN_DRAIN        0x00002000     // Open drain

//Polarity on interrupt signal
#define  IPOL_LOW          0x00000000     // Active low
#define  IPOL_HIGH         0x00004000     // Active high

//Function can wake up the system or is bypassed (OFF) in SLEEP mode
#define  WAKE_FFMT         0x00008000     // Frefall / Motion
#define  WAKE_FFMT_OFF     0x00000000     // Frefall / Motion is bypassed
#define  WAKE_PULSE        0x00010000     // Pulse
#define  WAKE_PULSE_OFF    0x00000000     // Pulse is bypassed
#define  WAKE_LNDPRT       0x00020000     // Landscape / Portrait
#define  WAKE_LNDPRT_OFF   0x00000000     // Landscape / Portrait is bypassed
#define  WAKE_TRANS        0x00080000     // Transient
#define  WAKE_TRANS_OFF    0x00000000     // Transient is bypassed

#define  INT1_DATAREADY    0x04100000     // Dataready interrupt is routed to INT1 
#define  INT2_DATAREADY    0x00100000     // Dataready interrupt is routed to INT2
#define  INT1_FFMT         0x08200000     // Frefall / Motion interrupt is routed to INT1
#define  INT2_FFMT         0x00200000     // Frefall / Motion interrupt is routed to INT2
#define  INT1_PULSE        0x10400000     // Pulse interrupt is routed to INT1
#define  INT2_PULSE        0x00400000     // Pulse interrupt is routed to INT2
#define  INT1_LNDPRT       0x20800000     // Lands / Portr interrupt is routed to INT1
#define  INT2_LNDPRT       0x00800000     // Lands / Portr interrupt is routed to INT2
#define  INT1_TRANS        0x41000000     // Transient interrupt is routed to INT1
#define  INT2_TRANS        0x01000000     // Transient interrupt is routed to INT2
#define  INT1_ASLP         0x82000000     // Auto sleep/wake interrupt is routed to INT1
#define  INT2_ASLP         0x02000000     // Auto sleep/wake interrupt is routed to INT2

///////////////////////////////////////////////////////////////////////////////
//                  Constants used for MMA8452_enTapDet() are:               //
///////////////////////////////////////////////////////////////////////////////
//Tap detection
#define  SINGLE_TAP        0b00000000     //Single tap detection activated
#define  DOUBLE_TAP        0b00000001     //Double tap detection activated
#define  SINGLE_DOUBLE     0b00000010     //Single and double tap detection activated

///////////////////////////////////////////////////////////////////////////////
//                  Constants used for MMA8452_xyz_conf() are:               //
///////////////////////////////////////////////////////////////////////////////
#define  HIGH_PASS_ON      0b00010000     //Enable high pass output data
#define  HIGH_PASS_OFF     0b00000000     //Output data high pass filtered
//Scale value range
#define  SET_2G            0b00000000     //2g
#define  SET_4G            0b00000001     //4g
#define  SET_8G            0b00000010     //8g

///////////////////////////////////////////////////////////////////////////////
//                  Constants used for MMA8452_filter() are:                 //
///////////////////////////////////////////////////////////////////////////////
//High pass filter cutoff frequency selection (for more information, refer to
//0x0F: HP_FILTER_CUTOFF High-Pass Filter Register in the datasheet
#define  SEL0              0b00000000     // Selection 0
#define  SEL1              0b00000001     // Selection 1
#define  SEL2              0b00000010     // Selection 2
#define  SEL3              0b00000011     // Selection 3
#define  HPF_EN            0b00000000     // High pass filter enabled
#define  HPF_BYP           0b00100000     // High pass filter bypassed
#define  LPF_EN            0b00010000     // Low pass filter enabled
#define  LPF_DIS           0b00000000     // Low pass filter disabled

///////////////////////////////////////////////////////////////////////////////
//                            Function protoypes                             //
///////////////////////////////////////////////////////////////////////////////
//    Write or read a register
void              writeReg(unsigned int reg, unsigned int data);
int               readReg(unsigned int reg);
//    Configuration and initialization
void              MMA8452_init();
void              MMA8452_active();
void              MMA8452_standby();
int               MMA8452_is_connected();
void              MMA8452_xyz_conf(int mask);
void              MMA8452_filter(int mask);
//    Read x,y and z axis
signed long int   MMA8452_x_read();
signed long int   MMA8452_y_read();
signed long int   MMA8452_z_read();
//    Portland and landscape orientation
void              MMA8452_enPortLand();
int               MMA8452_newOrientation();
int               MMA8452_LaPo();
int               MMA8452_BaFro();
//    Single and/or double tap detection
void              MMA8452_enTapDet(int mask);
int               MMA8452_getTap();
